
#include "GraspMemory.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/libraries/armem/core/error.h>
#include <RobotAPI/libraries/armem/server/MemoryRemoteGui.h>

#include <RobotAPI/libraries/GraspingUtility/aron/GraspCandidate.aron.generated.h>

namespace armarx::armem::server::grasp
{
    armarx::PropertyDefinitionsPtr GraspMemory::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs = new ComponentPropertyDefinitions(getConfigIdentifier());

        defs->topic(debugObserver);

        defs->optional(memoryName, "memory.Name", "Name of this memory server.");

        return defs;
    }


    std::string GraspMemory::getDefaultName() const
    {
        return "GraspMemory";
    }

    void GraspMemory::onInitComponent()
    {
        workingMemory().name() = memoryName;

        workingMemory().addCoreSegment("GraspCandidate",
                                     armarx::grasping::arondto::GraspCandidate::toAronType());
        workingMemory().addCoreSegment("BimanualGraspCandidate",
                                     armarx::grasping::arondto::BimanualGraspCandidate::toAronType());
    }

    void GraspMemory::onConnectComponent()
    {

        createRemoteGuiTab();
        RemoteGui_startRunningTask();

    }

    void GraspMemory::onDisconnectComponent()
    {
    }

    void GraspMemory::onExitComponent()
    {
    }



    // WRITING

    armem::data::CommitResult GraspMemory::commit(const armem::data::Commit& commit, const Ice::Current&)
    {
        // This function is overloaded to trigger the remote gui rebuild.
        armem::data::CommitResult result = ReadWritePluginUser::commit(commit);
        tab.rebuild = true;
        return result;
    }


    // READING

    // Inherited from Plugin



    // REMOTE GUI

    void GraspMemory::createRemoteGuiTab()
    {
        using namespace armarx::RemoteGui::Client;

        {

            tab.memoryGroup = armem::server::MemoryRemoteGui().makeGroupBox(workingMemory());
        }

        VBoxLayout root = {tab.memoryGroup, VSpacer()};
        RemoteGui_createTab(getName(), root, &tab);
    }


    void GraspMemory::RemoteGui_update()
    {
        if (tab.rebuild.exchange(false))
        {
            createRemoteGuiTab();
        }
    }

}
