#pragma once


#include <memory>

#include <ArmarXCore/interface/observers/ObserverInterface.h>
#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>
#include <RobotAPI/libraries/RobotAPIComponentPlugins/ArVizComponentPlugin.h>
#include <RobotAPI/libraries/armem/server/plugins/ReadWritePluginUser.h>

namespace armarx::armem::server::grasp
{
    /**
     * @defgroup Component-GraspMemory GraspMemory
     * @ingroup RobotAPI-Components
     * A description of the component GraspMemory.
     *
     * @class GraspMemory
     * @ingroup Component-GraspMemory
     * @brief Brief description of class GraspMemory.
     *
     * Detailed description of class GraspMemory.
     */
    class GraspMemory :
        virtual public armarx::Component
        , virtual public armem::server::ReadWritePluginUser
        , virtual public armarx::LightweightRemoteGuiComponentPluginUser
        , virtual public armarx::ArVizComponentPluginUser
    {
    public:

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;


    public:
        armem::data::CommitResult commit(const armem::data::Commit& commit, const Ice::Current&) override;


    public:
        void createRemoteGuiTab();
        void RemoteGui_update() override;


    protected:

        PropertyDefinitionsPtr createPropertyDefinitions() override;

        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;
        void onExitComponent() override;


    private:

        DebugObserverInterfacePrx debugObserver;


        std::string memoryName = "Grasp";



        struct RemoteGuiTab : RemoteGui::Client::Tab
        {
            std::atomic_bool rebuild = false;

            RemoteGui::Client::GroupBox memoryGroup;
        };
        RemoteGuiTab tab;

    };
}
