#pragma once

#include "MemoryBase.h"

namespace armarx::armem::server::ltm
{
    class BufferedMemoryItem
    {
    public:
        BufferedMemoryItem() = default;
        BufferedMemoryItem(const MemoryID& id) :
            buffer(id)
        {
        }

        virtual ~BufferedMemoryItem() = default;

        armem::wm::Memory getBuffer() const
        {
            return buffer;
        }

        void directlyStore(const armem::wm::Memory& memory)
        {
            TIMING_START(LTM_Memory_DirectlyStore);
             _directlyStore(memory);
            TIMING_END(LTM_Memory_DirectlyStore);
        }

        void storeBuffer()
        {
            if (buffer.empty())
            {
                ARMARX_INFO << deactivateSpam() << "Cannot store an empty buffer. Ignoring.";
                return;
            }
            this->directlyStore(buffer);
            buffer.clear();
        }

    protected:
        virtual void _directlyStore(const armem::wm::Memory& memory) = 0;

    protected:

        /// Internal memory for data consolidated from wm to ltm (buffer)
        /// The to-put-to-ltm buffer (contains data in plain text)
        /// This buffer may still be filtered.
        /// This means that it is not guaranteed that all data in the buffer will be stored in the ltm
        armem::wm::Memory buffer;

    };
} // namespace armarx::armem::server::ltm
