#pragma once

#include <functional>

// BaseClass
#include "MemoryItem.h"

// ChildType
#include "CoreSegmentBase.h"

// ArmarX
#include <ArmarXCore/core/application/properties/Properties.h>
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/LoggingUtil.h>

#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem/core/wm/aron_conversions.h>
#include <RobotAPI/libraries/armem/core/operations.h>
#include <RobotAPI/libraries/armem/server/wm/memory_definitions.h>


namespace armarx::armem::server::ltm
{
    /// @brief Interface functions for the longterm memory classes
    template <class _CoreSegmentT>
    class MemoryBase : public MemoryItem
    {
    public:
        using CoreSegmentT = _CoreSegmentT;

        using MemoryItem::MemoryItem;

        /// return the full ltm as a wm::Memory with only references
        /// the ltm may be huge, use with caution
        void loadAll(armem::wm::Memory& memory)
        {
            TIMING_START(LTM_Memory_LoadAll);
            _loadAll(memory);
            TIMING_END(LTM_Memory_LoadAll);
        }

        /// convert the references of the input into a wm::Memory
        void load(armem::wm::Memory& memory)
        {
            TIMING_START(LTM_Memory_Load);
            _load(memory);
            TIMING_END(LTM_Memory_Load);
        }

        /// append a wm::Memory instance to the ltm
        void store(const armem::wm::Memory& memory)
        {
            TIMING_START(LTM_Memory_Append);
            if (pipeline.memFilter->accept(memory))
            {
                _store(memory);
            }
            else
            {
                ARMARX_WARNING << deactivateSpam() << "Ignoring to put a Memory into the LTM because it got filtered.";
            }
            TIMING_END(LTM_Memory_Append);
        }

        /// append a wm::Memory instance to the ltm
        void store(const armem::server::wm::Memory& serverMemory)
        {
            wm::Memory memory;
            memory.update(armem::toCommit(serverMemory));
            this->store(memory);
        }

        /// iterate over all core segments of this ltm
        virtual bool forEachCoreSegment(std::function<void(CoreSegmentT&)>&& func) const = 0;

        /// find core segment
        virtual std::shared_ptr<CoreSegmentT> findCoreSegment(const std::string&) const = 0;

        /// parameters
        virtual void createPropertyDefinitions(PropertyDefinitionsPtr& properties, const std::string& prefix)
        {}

        /// get level name
        static std::string getLevelName()
        {
            return "LT-Memory";
        }

    protected:
        virtual void _loadAll(armem::wm::Memory& memory) = 0;
        virtual void _load(armem::wm::Memory& memory) = 0;
        virtual void _store(const armem::wm::Memory& memory) = 0;

    public:
        bool enabled = false;

    protected:
        mutable std::recursive_mutex ltm_mutex;
    };
} // namespace armarx::armem::server::ltm
