#pragma once

#include <map>
#include <mutex>
#include <optional>
#include <string>

#include "../filter/Filter.h"
#include "../extractor/Extractor.h"
#include "../converter/dict/Converter.h"
#include "../converter/image/Converter.h"

#include <RobotAPI/libraries/armem/core/MemoryID.h>

namespace armarx::armem::server::ltm
{
    struct LongTermMemoryPipeline
    {
        MemoryFilterPtr memFilter;
        SnapshotFilterPtr snapFilter;
        DictConverterPtr dictConverter;
        ImageConverterPtr imgConverter;
        ExtractorPtr extractor;
    };

    /// @brief Interface functions for the longterm memory classes
    class MemoryItem
    {
    public:
        MemoryItem() = default;
        MemoryItem(const MemoryID&);
        MemoryItem(const MemoryID&, const LongTermMemoryPipeline&);
        virtual ~MemoryItem() = default;

        MemoryID id() const;
        std::string name() const;

        virtual void setMemoryID(const MemoryID&);

    protected:
        LongTermMemoryPipeline pipeline;

    private:
        MemoryID _id;
    };
} // namespace armarx::armem::server::ltm
