#pragma once

#include <filesystem>

// Base Class
#include <RobotAPI/libraries/armem/server/ltm/base/detail/BufferedMemoryBase.h>
#include "detail/DiskStorage.h"

// Segmnet Type
#include "CoreSegment.h"


namespace armarx::armem::server::ltm::disk
{
    /// @brief A memory storing data in mongodb (needs 'armarx memory start' to start the mongod instance)
    class Memory :
            public MemoryBase<CoreSegment>,
            public BufferedMemoryItem,
            public DiskMemoryItem
    {
    public:
        using Base = MemoryBase<CoreSegment>;

        Memory();
        Memory(const std::filesystem::path&);

        void setMemoryID(const MemoryID& id) override;

        void createPropertyDefinitions(PropertyDefinitionsPtr& properties, const std::string& prefix) override;

        bool forEachCoreSegment(std::function<void(CoreSegment&)>&& func) const override;

        std::shared_ptr<CoreSegment> findCoreSegment(const std::string&) const override;

    protected:
        void _loadAll(armem::wm::Memory&) override;
        void _load(armem::wm::Memory&) override;
        void _store(const armem::wm::Memory&) override;
        void _directlyStore(const armem::wm::Memory&) override;

        std::string getExpectedFolderName() const override;

    };
} // namespace armarx::armem::server::ltm::disk
