#pragma once

#include <RobotAPI/libraries/armem/server/forward_declarations.h>
#include <RobotAPI/libraries/armem/server/wm/memory_definitions.h>
#include <RobotAPI/libraries/aron/core/type/variant/forward_declarations.h>

#include <ArmarXCore/core/application/properties/forward_declarations.h>
#include <ArmarXCore/core/logging/Logging.h>

#include <string>


namespace armarx::armem::server::segment
{

    /**
     * @brief Specialized management of a core segment.
     */
    class SpecializedSegment :
        public armarx::Logging
    {
    public:

        SpecializedSegment(
            server::MemoryToIceAdapter& iceMemory,
            aron::type::ObjectPtr aronType = nullptr,
            const std::string& defaultCoreSegmentName = "",
            int64_t defaultMaxHistorySize = -1);

        virtual ~SpecializedSegment();


        virtual void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");
        virtual void init();
        // void connect();


        template <class FunctionT>
        auto doLocked(FunctionT&& function) const
        {
            return coreSegment->doLocked(function);
        }


    protected:

        void setDefaultCoreSegmentName(const std::string& coreSegmentName);
        void setDefaultMaxHistorySize(int64_t maxHistorySize);
        void setAronType(aron::type::ObjectPtr aronType);


    protected:

        server::MemoryToIceAdapter& iceMemory;
        server::wm::CoreSegment* coreSegment = nullptr;
        aron::type::ObjectPtr aronType;

        struct Properties
        {
            std::string coreSegmentName = "";
            int64_t maxHistorySize = -1;
        };
        Properties properties;


    };

}
