/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <vector>

#include <RobotAPI/interface/units/LaserScannerUnit.h>
#include <RobotAPI/libraries/armem/core/Time.h>

namespace armarx::armem
{

    struct SensorHeader
    {
        std::string agent;
        std::string frame;
        armem::Time timestamp;
    };

    struct LaserScanStamped
    {
        SensorHeader header;
        LaserScan data;
    };

    // template<typename _ValueT = float>
    struct OccupancyGrid
    {
        float resolution; // [mm]

        std::string frame;
        Eigen::Affine3f pose;

        // using ValueType = _ValueT;
        using CellType = float;
        using Grid     = Eigen::Array<CellType, Eigen::Dynamic, Eigen::Dynamic>;

        Grid grid;
    };


} // namespace armarx::armem
