/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "Factory.h"

// ArmarX
#include "All.h"
#include "enum/All.h"

namespace armarx::aron::codegenerator::cpp
{
    // Access method
    std::unique_ptr<Generator> GeneratorFactory::create(const type::Variant& n, const Path& path) const
    {
        auto desc = n.getDescriptor();
        switch(desc)
        {
            case type::Descriptor::eList: return std::make_unique<generator::List>(dynamic_cast<const type::List&>(n));
            case type::Descriptor::eDict: return std::make_unique<generator::Dict>(dynamic_cast<const type::Dict&>(n));
            case type::Descriptor::eObject: return std::make_unique<generator::Object>(dynamic_cast<const type::Object&>(n));
            case type::Descriptor::eTuple: return std::make_unique<generator::Tuple>(dynamic_cast<const type::Tuple&>(n));
            case type::Descriptor::ePair: return std::make_unique<generator::Pair>(dynamic_cast<const type::Pair&>(n));
            case type::Descriptor::eNDArray: return std::make_unique<generator::NDArray>(dynamic_cast<const type::NDArray&>(n));
            case type::Descriptor::eMatrix: return std::make_unique<generator::Matrix>(dynamic_cast<const type::Matrix&>(n));
            case type::Descriptor::eQuaternion: return std::make_unique<generator::Quaternion>(dynamic_cast<const type::Quaternion&>(n));
            case type::Descriptor::eImage: return std::make_unique<generator::Image>(dynamic_cast<const type::Image&>(n));
            case type::Descriptor::ePointCloud: return std::make_unique<generator::PointCloud>(dynamic_cast<const type::PointCloud&>(n));
            case type::Descriptor::ePosition: return std::make_unique<generator::Position>(dynamic_cast<const type::Position&>(n));
            case type::Descriptor::eOrientation: return std::make_unique<generator::Orientation>(dynamic_cast<const type::Orientation&>(n));
            case type::Descriptor::ePose: return std::make_unique<generator::Pose>(dynamic_cast<const type::Pose&>(n));
            case type::Descriptor::eIntEnum: return std::make_unique<generator::IntEnum>(dynamic_cast<const type::IntEnum&>(n));
            case type::Descriptor::eInt: return std::make_unique<generator::Int>(dynamic_cast<const type::Int&>(n));
            case type::Descriptor::eLong: return std::make_unique<generator::Long>(dynamic_cast<const type::Long&>(n));
            case type::Descriptor::eFloat: return std::make_unique<generator::Float>(dynamic_cast<const type::Float&>(n));
            case type::Descriptor::eDouble: return std::make_unique<generator::Double>(dynamic_cast<const type::Double&>(n));
            case type::Descriptor::eString: return std::make_unique<generator::String>(dynamic_cast<const type::String&>(n));
            case type::Descriptor::eBool: return std::make_unique<generator::Bool>(dynamic_cast<const type::Bool&>(n));
            case type::Descriptor::eTime: return std::make_unique<generator::Time>(dynamic_cast<const type::Time&>(n));
            default: throw error::ValueNotValidException(__PRETTY_FUNCTION__, "Cannot create a generator", std::to_string((int) desc), path);;
        }
    }

}
