#include "DataVisitor.h"

#include <SimoxUtility/meta/type_name.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::aron::visitor
{


    DataVisitor::DataVisitor()
    {
    }

    DataVisitor::~DataVisitor()
    {
    }


    bool DataVisitor::applyTo(data::dto::GenericDataPtr& data)
    {
        ARMARX_CHECK_NOT_NULL(data);
        return applyTo(*DataNavigator::FromAronDTO(data));
    }

    bool DataVisitor::applyTo(DataVisitor::DataNavigator& navigator)
    {
        return this->applyTo("", navigator);
    }

    bool DataVisitor::applyTo(const std::string& key, DataNavigator& navigator)
    {
        auto handleCollection = [this, key](auto & n)
        {
            if (!visitEnter(key, n))
            {
                return false;
            }
            if (!applyToChildren(n))
            {
                return false;
            }
            if (!visitExit(key, n))
            {
                return false;
            }
            return true;
        };

        if (auto n = dynamic_cast<DictDataNavigator*>(&navigator))
        {
            return handleCollection(*n);
        }
        else if (auto n = dynamic_cast<ListDataNavigator*>(&navigator))
        {
            return handleCollection(*n);
        }

        ARMARX_CHECK_EQUAL(navigator.childrenSize(), 0)
                << simox::meta::get_type_name(navigator);

        if (auto n = dynamic_cast<BoolDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<DoubleDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<FloatDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<IntDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<LongDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<NDArrayDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<StringDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        ARMARX_CHECK(false) << "Unhandled AronDataNavigatorType '" << simox::meta::get_type_name(navigator) << "'.";
    }

    bool DataVisitor::applyToChildren(DictDataNavigator& navigator)
    {
        for (const std::string& key : navigator.getAllKeys())
        {
            auto child = navigator.getElement(key);
            if (child)
            {
                if (!applyTo(key, *child))
                {
                    return false;
                }
            }
        }
        return true;
    }

    bool DataVisitor::applyToChildren(ListDataNavigator& navigator)
    {
        size_t i = 0;
        for (auto& child : navigator.getChildren())
        {
            if (child)
            {
                if (!applyTo(std::to_string(i), *child))
                {
                    return false;
                }
            }
            ++i;
        }
        return true;
    }

}
