/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/core/type/rw/Reader.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/type/variant/Variant.h>


namespace armarx::aron::type::reader
{
    class NavigatorReader :
        public ReaderInterface<aron::type::VariantPtr>
    {
    public:
        // constructors
        NavigatorReader() = default;

        void readObject(const aron::type::VariantPtr& input, std::string& name, std::map<std::string, aron::type::VariantPtr>& memberTypes, type::Maybe& maybe) override;
        void readList(const aron::type::VariantPtr& input, aron::type::VariantPtr& acceptedType, type::Maybe& maybe) override;
        void readDict(const aron::type::VariantPtr& input, aron::type::VariantPtr& acceptedType, type::Maybe& maybe) override;
        void readTuple(const aron::type::VariantPtr& input, std::vector<aron::type::VariantPtr>& acceptedTypes, type::Maybe& maybe) override;
        void readPair(const aron::type::VariantPtr& input, std::pair<aron::type::VariantPtr, aron::type::VariantPtr>& acceptedTypes, type::Maybe& maybe) override;

        void readNDArray(const aron::type::VariantPtr& input, int& ndim, type::ndarray::ElementType& type, type::Maybe& maybe) override;
        void readMatrix(const aron::type::VariantPtr& input, int& rows, int& cols, type::matrix::ElementType& type, type::Maybe& maybe) override;
        void readQuaternion(const aron::type::VariantPtr& input, type::quaternion::ElementType& type, type::Maybe& maybe) override;
        void readPointCloud(const aron::type::VariantPtr& input, type::pointcloud::VoxelType& type, type::Maybe& maybe) override;
        void readImage(const aron::type::VariantPtr& input, type::image::PixelType& type, type::Maybe& maybe) override;
        void readPosition(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readOrientation(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readPose(const aron::type::VariantPtr& input, type::Maybe& maybe) override;

        void readIntEnum(const aron::type::VariantPtr& input, std::string& name, std::map<std::string, int>& acceptedValues, type::Maybe& maybe) override;

        void readInt(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readLong(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readFloat(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readDouble(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readString(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readBool(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
        void readTime(const aron::type::VariantPtr& input, type::Maybe& maybe) override;
    };
}
