/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD / STL
#include <memory>
#include <stack>
#include <sstream>

// Simox
#include <SimoxUtility/json.h>

// Base Class
#include <RobotAPI/libraries/aron/core/type/rw/Writer.h>

namespace armarx::aron::type::writer
{
    class NlohmannJSONWriter :
        public WriterInterface<nlohmann::json>
    {
    public:
        NlohmannJSONWriter() = default;

        virtual nlohmann::json writeObject(const std::string& name, const type::Maybe maybe, const std::map<std::string, nlohmann::json>& memberTypes, const std::optional<nlohmann::json>& extends = std::nullopt) override;
        virtual nlohmann::json writeList(const type::Maybe maybe, const nlohmann::json& acceptedType) override;
        virtual nlohmann::json writeDict(const type::Maybe maybe, const nlohmann::json& acceptedType) override;
        virtual nlohmann::json writePair(const type::Maybe maybe, const nlohmann::json& acceptedType1, const nlohmann::json& acceptedType2) override;
        virtual nlohmann::json writeTuple(const type::Maybe maybe, const std::vector<nlohmann::json>& acceptedTypes) override;

        virtual nlohmann::json writeNDArray(const int ndim, const type::ndarray::ElementType, const type::Maybe maybe) override;
        virtual nlohmann::json writeMatrix(const int rows, const int cols, const type::matrix::ElementType type, const type::Maybe maybe) override;
        virtual nlohmann::json writeQuaternion(const type::quaternion::ElementType, const type::Maybe maybe) override;
        virtual nlohmann::json writeImage(const type::image::PixelType, const type::Maybe maybe) override;
        virtual nlohmann::json writePointCloud(const type::pointcloud::VoxelType, const type::Maybe maybe) override;
        virtual nlohmann::json writePosition(const type::Maybe maybe) override;
        virtual nlohmann::json writeOrientation(const type::Maybe maybe) override;
        virtual nlohmann::json writePose(const type::Maybe maybe) override;

        virtual nlohmann::json writeIntEnum(const std::string& name, const std::map<std::string, int>& acceptedValues, const type::Maybe maybe) override;

        virtual nlohmann::json writeInt(const type::Maybe maybe) override;
        virtual nlohmann::json writeLong(const type::Maybe maybe) override;
        virtual nlohmann::json writeFloat(const type::Maybe maybe) override;
        virtual nlohmann::json writeDouble(const type::Maybe maybe) override;
        virtual nlohmann::json writeString(const type::Maybe maybe) override;
        virtual nlohmann::json writeBool(const type::Maybe maybe) override;
        virtual nlohmann::json writeTime(const type::Maybe maybe) override;
    };
}
