/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <map>
#include <string>
#include <vector>

#include "../RecursiveVisitor.h"
#include "../../variant/All.h"

namespace armarx::aron::type
{
    /**
     * @brief The VariantVisitor struct. Already implements the method to get the descriptor of an aron variant.
     */
    struct VariantVisitor : virtual public Visitor<const type::VariantPtr>
    {
        virtual type::Descriptor getDescriptor(Input& n) override
        {
            return n->getDescriptor();
        }
        virtual ~VariantVisitor() = default;
    };

    /**
     * @brief The RecursiveVariantVisitor struct. Already implements the methods to get the descriptor and children of an aron variant
     */
    struct RecursiveVariantVisitor : virtual public RecursiveVisitor<const type::VariantPtr>
    {
        virtual type::Descriptor getDescriptor(Input& n) override
        {
            return n->getDescriptor();
        }

        virtual std::map<std::string, type::VariantPtr> getObjectAcceptedTypes(Input& t) override
        {
            auto o = type::Object::DynamicCastAndCheck(t);
            return o->getMemberTypes();
        }
        virtual type::VariantPtr getDictAcceptedType(Input& t) override
        {
            auto o = type::Dict::DynamicCastAndCheck(t);
            return o->getAcceptedType();
        }
        virtual type::VariantPtr getListAcceptedType(Input& t) override
        {
            auto o = type::List::DynamicCastAndCheck(t);
            return o->getAcceptedType();
        }
        virtual std::pair<type::VariantPtr, type::VariantPtr> getPairAcceptedTypes(Input& t) override
        {
            auto o = type::Pair::DynamicCastAndCheck(t);
            return o->getAcceptedTypes();
        }
        virtual std::vector<type::VariantPtr> getTupleAcceptedTypes(Input& t) override
        {
            auto o = type::Tuple::DynamicCastAndCheck(t);
            return o->getAcceptedTypes();
        }

        virtual ~RecursiveVariantVisitor() = default;
    };
}
